@@ -126,17 +126,22 @@ public class FileSystemBackedTableMetadata implements HoodieTableMetadata {
   }
 
   @Override
-  public Option<String> getUpdateTime() {
+  public Option<String> getSyncedInstantTime() {
     throw new UnsupportedOperationException();
   }
 
   @Override
-  public boolean isInSync() {
-    return true;
+  public Option<String> getLatestCompactionTime() {
+    throw new UnsupportedOperationException();
   }
 
   @Override
   public void close() throws Exception {
     // no-op
   }
+
+  @Override
+  public void reset() {
+    // no-op
+  }
 }
