@@ -46,7 +46,7 @@ import java.util.List;
  * SQL API.
  *
  * <p>Note: Changes in this class need to be kept in sync with the corresponding runtime classes
- * {@link org.apache.flink.formats.avro.AvroRowDeserializationSchema} and {@link org.apache.flink.formats.avro.AvroRowSerializationSchema}.
+ * {@code org.apache.flink.formats.avro.AvroRowDeserializationSchema} and {@code org.apache.flink.formats.avro.AvroRowSerializationSchema}.
  *
  * <p>NOTE: reference from Flink release 1.12.0, should remove when Flink version upgrade to that.
  */
@@ -294,7 +294,7 @@ public class AvroSchemaConverter {
     }
   }
 
-  private static LogicalType extractValueTypeToAvroMap(LogicalType type) {
+  public static LogicalType extractValueTypeToAvroMap(LogicalType type) {
     LogicalType keyType;
     LogicalType valueType;
     if (type instanceof MapType) {
