@@ -20,6 +20,8 @@ package org.apache.spark.sql.hudi
 import org.apache.hudi.common.table.HoodieTableMetaClient
 import org.apache.hudi.exception.HoodieDuplicateKeyException
 
+import java.io.File
+
 class TestInsertTable extends TestHoodieSqlBase {
 
   test("Test Insert Into") {
@@ -238,29 +240,46 @@ class TestInsertTable extends TestHoodieSqlBase {
       )
       typeAndValue.foreach { case (partitionType, partitionValue) =>
         val tableName = generateTableName
-        // Create table
-        spark.sql(
-          s"""
-             |create table $tableName (
-             |  id int,
-             |  name string,
-             |  price double,
-             |  dt $partitionType
-             |) using hudi
-             | tblproperties (primaryKey = 'id')
-             | partitioned by (dt)
-             | location '${tmp.getCanonicalPath}/$tableName'
-       """.stripMargin)
-        spark.sql(s"insert into $tableName partition(dt = $partitionValue) select 1, 'a1', 10")
-        spark.sql(s"insert into $tableName select 2, 'a2', 10, $partitionValue")
-        checkAnswer(s"select id, name, price, cast(dt as string) from $tableName order by id")(
-          Seq(1, "a1", 10, removeQuotes(partitionValue).toString),
-          Seq(2, "a2", 10, removeQuotes(partitionValue).toString)
-        )
+        validateDifferentTypesOfPartitionColumn(tmp, partitionType, partitionValue, tableName)
       }
     }
   }
 
+  test("Test TimestampType Partition Column With Consistent Logical Timestamp Enabled") {
+    withTempDir { tmp =>
+      val typeAndValue = Seq(
+        ("timestamp", "'2021-05-20 00:00:00'"),
+        ("date", "'2021-05-20'")
+      )
+      typeAndValue.foreach { case (partitionType, partitionValue) =>
+        val tableName = generateTableName
+        spark.sql(s"set hoodie.datasource.write.keygenerator.consistent.logical.timestamp.enabled=true")
+        validateDifferentTypesOfPartitionColumn(tmp, partitionType, partitionValue, tableName)
+      }
+    }
+  }
+
+  private def validateDifferentTypesOfPartitionColumn(tmp: File, partitionType: String, partitionValue: Any, tableName: String) = {
+    spark.sql(
+      s"""
+         |create table $tableName (
+         |  id int,
+         |  name string,
+         |  price double,
+         |  dt $partitionType
+         |) using hudi
+         | tblproperties (primaryKey = 'id')
+         | partitioned by (dt)
+         | location '${tmp.getCanonicalPath}/$tableName'
+       """.stripMargin)
+    spark.sql(s"insert into $tableName partition(dt = $partitionValue) select 1, 'a1', 10")
+    spark.sql(s"insert into $tableName select 2, 'a2', 10, $partitionValue")
+    checkAnswer(s"select id, name, price, cast(dt as string) from $tableName order by id")(
+      Seq(1, "a1", 10, removeQuotes(partitionValue).toString),
+      Seq(2, "a2", 10, removeQuotes(partitionValue).toString)
+    )
+  }
+
   test("Test insert for uppercase table name") {
     withTempDir{ tmp =>
       val tableName = s"H_$generateTableName"
@@ -351,6 +370,38 @@ class TestInsertTable extends TestHoodieSqlBase {
     spark.sql("set hoodie.sql.insert.mode = upsert")
   }
 
+
+  test("Test Insert timestamp when 'spark.sql.datetime.java8API.enabled' enables") {
+    try {
+      // enable spark.sql.datetime.java8API.enabled
+      // and use java.time.Instant to replace java.sql.Timestamp to represent TimestampType.
+      spark.conf.set("spark.sql.datetime.java8API.enabled", value = true)
+
+      val tableName = generateTableName
+      spark.sql(
+        s"""
+           |create table $tableName (
+           |  id int,
+           |  name string,
+           |  price double,
+           |  dt timestamp
+           |)
+           |using hudi
+           |partitioned by(dt)
+           |options(type = 'cow', primaryKey = 'id')
+           |""".stripMargin
+      )
+
+      spark.sql(s"insert into $tableName values (1, 'a1', 10, cast('2021-05-07 00:00:00' as timestamp))")
+      checkAnswer(s"select id, name, price, cast(dt as string) from $tableName")(
+        Seq(1, "a1", 10, "2021-05-07 00:00:00")
+      )
+
+    } finally {
+      spark.conf.set("spark.sql.datetime.java8API.enabled", value = false)
+    }
+  }
+
   test("Test bulk insert") {
     spark.sql("set hoodie.sql.insert.mode = non-strict")
     withTempDir { tmp =>
